/*
	Copyright (C) 2022 lewis.

	This file is part of ladspa-tool-kit stream extension

	ladspa-tool-kit stream extension is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	ladspa-tool-kit stream extension is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
	See the GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with ladspa-tool-kit stream extension.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef __LTKSTREAM_H__
#define __LTKSTREAM_H__

#include <sys/stat.h>
#include <LTK/instance.h>

/* Members */
/* Man paged function */
#define LTK_STREAM_READ				0 /* ssize_t *size, void *data, int blocking */
/*	Name complement : The member 1 does this
	Description :
		Describe here the member one goal behaviour.
*/
/* Man paged function */
#define LTK_STREAM_WRITE				1 /* ssize_t *size, void *data, int blocking */
/*	Name complement : The member 2 does that
	Description :
		Describe here the member two goal behaviour.
*/
/* Man paged function */
#define LTK_STREAM_PRINT				2 /* ssize_t *size, char *format, ... */
/*	Name complement : The member 2 does that
	Description :
		Describe here the member two goal behaviour.
*/
/* Man paged function */
#define LTK_STREAM_CALLBACK			3 /* ltk_callback_function_t function, void *data, uint32_t events */
/*	Name complement : Subscribe/Unsubscribe the fd in the callback object
	Description :
		Given function, LTK_CALLBACK_SET mark the fd attached to the callback as triggable.
		Therefore, any subsequent call to LTKSync may call the given function argument.
		A data pointer may be given to the callback object using the data argument.
*/

/* Attributes */
typedef enum ltk_callback_free_behaviour_e {
	LTK_STREAM_FREE_BEHAVIOUR_EPOLL_REMOVE = 0, /* Default behaviour */
	LTK_STREAM_FREE_BEHAVIOUR_CLOSE_ONLY = 1 /* Don't treat epoll on free */
} ltk_callback_free_behaviour_t;

typedef void (*ltk_callback_function_t) (void *data);

typedef struct ltk_stream_s {
	int _fd;						/* Stream descriptor */
	struct stat _status;			/* Stream stat (see man 2 stat) */
	ltk_callback_free_behaviour_t _free_behaviour;
	ltk_callback_function_t _callback_function;
	void *_callback_data;
} ltk_stream_t;

LTKClassTypeDefinition_t *ltk_stream_ctd();
ltk_stream_t *ltk_stream_new(int fd);

/* Man paged function */
void LTKSync(int timeout);
/*	Name complement : Stop current process execution waiting an event to happen
	Description :
		Wait for currently created callbacks to trigger.
		This call uses epoll_wait() to wait for an event to occur on a registered fd.
		As each fd has been registered given an attached ltk_callback structure,
		the given structure is used to retreive what function to call
			and which data to give it as parameter
		The timeout argument is given as is to the epoll_wait() call
			so refer to epoll_wait(2) for more information
	See also : epoll_wait(2)
*/

#endif /* __LTKSTREAM_H__ */

